@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.63.4.1 2005/10/01 07:38:27 chpe Exp $
+ *  $Id: ephy-location-entry.c,v 1.71 2006/01/29 21:47:22 chpe Exp $
  */
 
 #include "config.h"
@@ -65,6 +65,8 @@ struct _EphyLocationEntryPrivate
 	GtkWidget *lock_ebox;
 	GtkWidget *lock;
 	GdkPixbuf *favicon;
+	GdkColor secure_bg_colour;
+	GdkColor secure_fg_colour;
 
 	char *before_completion;
 
@@ -74,8 +76,11 @@ struct _EphyLocationEntryPrivate
 	guint relevance_col;
 
 	guint hash;
+
 	guint user_changed : 1;
 	guint original_address : 1;
+	guint secure : 1;
+	guint apply_colours : 1;
 };
 
 static const struct
@@ -99,6 +104,8 @@ static const GtkTargetEntry url_drag_types [] =
 	{ EPHY_DND_TEXT_TYPE,       0, 2 }
 };
 
+static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+
 static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
 
@@ -161,6 +168,61 @@ ephy_location_entry_set_tooltip (GtkToolItem *tool_item,
 	return TRUE;
 }
 
+static void
+ephy_location_entry_style_set (GtkWidget *widget,
+			       GtkStyle *previous_style)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (widget);
+	EphyLocationEntryPrivate *priv = entry->priv;
+	GtkSettings *settings;
+	GdkColor *bg_colour = NULL, *fg_colour = NULL;
+	char *theme;
+	gboolean is_a11y_theme;
+
+	if (GTK_WIDGET_CLASS (parent_class)->style_set)
+	{
+		GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+	}
+
+	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
+	g_object_get (settings, "gtk-theme-name", &theme, NULL);
+	is_a11y_theme = strncmp (theme, "HighContrast", strlen ("HighContrast")) == 0 ||
+			strncmp (theme, "LowContrast", strlen ("LowContrast")) == 0;
+	g_free (theme);
+
+	gtk_widget_style_get (widget,
+			      "secure-fg-color", &fg_colour,
+			      "secure-bg-color", &bg_colour,
+			      NULL);
+
+	/* We only use the fallback colours when we don't have an a11y theme */
+	priv->apply_colours = !is_a11y_theme || (fg_colour != NULL && bg_colour != NULL);
+
+	if (fg_colour != NULL)
+	{
+		priv->secure_fg_colour = *fg_colour;
+		gdk_color_free (fg_colour);
+	}
+	else
+	{
+		/* FIXME: fall back to black text maybe? */
+		priv->secure_fg_colour = widget->style->text[GTK_STATE_NORMAL];
+	}
+
+	if (bg_colour != NULL)
+	{
+		priv->secure_bg_colour = *bg_colour;
+		gdk_color_free (bg_colour);
+	}
+	else
+	{
+		priv->secure_bg_colour = fallback_bg_colour;
+	}
+
+	/* Apply the new style */
+	ephy_location_entry_set_secure (entry, priv->secure);
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -181,12 +243,15 @@ static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkToolItemClass *tool_item_class = GTK_TOOL_ITEM_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = ephy_location_entry_finalize;
 
+	widget_class->style_set = ephy_location_entry_style_set;
+
 	tool_item_class->set_tooltip = ephy_location_entry_set_tooltip;
 
 	signals[USER_CHANGED] = g_signal_new (
@@ -229,6 +294,19 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
+        gtk_widget_class_install_style_property (widget_class,
+                                                 g_param_spec_boxed ("secure-bg-color",
+                                                                     "Secure background colour",
+                                                                     "Background colour to use for secure sites",
+                                                                     GDK_TYPE_COLOR,
+                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+        gtk_widget_class_install_style_property (widget_class,
+                                                 g_param_spec_boxed ("secure-fg-color",
+                                                                     "Secure foreground Colour",
+                                                                     "Foreground colour to use for secure sites",
+                                                                     GDK_TYPE_COLOR,
+                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
 }
@@ -302,7 +380,8 @@ entry_key_press_cb (GtkEntry *entry,
 	if ((event->keyval == GDK_Return ||
 	     event->keyval == GDK_KP_Enter ||
 	     event->keyval == GDK_ISO_Enter) &&
-	    state == GDK_CONTROL_MASK)
+	    (state == GDK_CONTROL_MASK ||
+	     state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
 	{
 		gtk_im_context_reset (entry->im_context);
 
@@ -310,7 +389,8 @@ entry_key_press_cb (GtkEntry *entry,
 
 		return TRUE;
 	}
-	else if (event->keyval == GDK_Escape && state == 0)
+
+	if (event->keyval == GDK_Escape && state == 0)
 	{
 		ephy_location_entry_reset (lentry);
 		/* don't return TRUE since we want to cancel the autocompletion popup too */
@@ -538,6 +618,148 @@ each_url_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 	g_free (title);
 }
 
+#define DRAG_ICON_LAYOUT_BORDER		2
+#define DRAG_ICON_ICON_SPACING		DRAG_ICON_LAYOUT_BORDER * 2
+#define DRAG_ICON_MAX_WIDTH_CHARS	32
+
+static GdkPixmap *
+favicon_create_drag_pixmap (EphyLocationEntry *entry,
+			    GtkWidget *widget)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	char *title = NULL, *address = NULL;
+	GString *text;
+	GdkDrawable *drawable;
+	PangoContext *context;
+	PangoLayout  *layout;
+	PangoFontMetrics *metrics;
+	int pixmap_height, pixmap_width;
+	int layout_width, layout_height;
+	int icon_width = 0, icon_height = 0, offset_x = 0;
+	int char_width;
+
+	g_signal_emit (entry, signals[GET_LOCATION], 0, &address);
+	g_signal_emit (entry, signals[GET_TITLE], 0, &title);
+	if (address == NULL || title == NULL) return NULL;
+
+	/* Compute text */
+	title = g_strstrip (title);
+
+	text = g_string_sized_new (strlen (address) + strlen (title) + 2);
+	if (title[0] != '\0')
+	{
+		g_string_append (text, title);
+		g_string_append (text, "\n");
+	}
+
+	if (address[0] != '\0')
+	{
+		g_string_append (text, address);
+	}
+
+	/* Now build the pixmap */
+
+	if (priv->favicon != NULL)
+	{
+		icon_width = gdk_pixbuf_get_width (priv->favicon);
+		icon_height = gdk_pixbuf_get_height (priv->favicon);
+	}
+
+	context = gtk_widget_get_pango_context (widget);
+	layout = pango_layout_new (context);
+
+	context = gtk_widget_get_pango_context (widget);
+	metrics = pango_context_get_metrics (context,
+					     widget->style->font_desc,
+					     pango_context_get_language (context));
+
+	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
+	pango_font_metrics_unref (metrics);
+
+	pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
+	pango_layout_set_width (layout, char_width * DRAG_ICON_MAX_WIDTH_CHARS);
+	pango_layout_set_text (layout, text->str, text->len);
+
+	pango_layout_get_size (layout, &layout_width, &layout_height);
+
+	pixmap_width  = layout_width  / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+
+	if (priv->favicon != NULL)
+	{
+		offset_x = icon_width + 2 * DRAG_ICON_ICON_SPACING;
+		pixmap_width += offset_x;
+		pixmap_height = MAX (layout_height / PANGO_SCALE, icon_height) + DRAG_ICON_LAYOUT_BORDER * 2;
+	}
+	else
+	{
+		pixmap_height = layout_height / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
+	}
+
+	drawable = gdk_pixmap_new (widget->window,
+				   pixmap_width  + 2,
+				   pixmap_height + 2,
+				   -1);
+
+	gdk_draw_rectangle (drawable,
+			    widget->style->base_gc [GTK_WIDGET_STATE (widget)],
+			    TRUE,
+			    0, 0,
+			    pixmap_width + 1,
+			    pixmap_height + 1);
+
+
+	if (priv->favicon != NULL)
+	{
+		gdk_draw_pixbuf (drawable,
+				 widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+				 priv->favicon,
+				 0, 0, 
+				 1 + DRAG_ICON_LAYOUT_BORDER + DRAG_ICON_ICON_SPACING,
+				 1 + DRAG_ICON_LAYOUT_BORDER + (pixmap_height - icon_height) / 2,
+				 -1, -1,
+				 GDK_RGB_DITHER_NONE, 0, 0);
+	}
+
+	gdk_draw_layout (drawable,
+			 widget->style->text_gc [GTK_WIDGET_STATE (widget)],
+			 1 + DRAG_ICON_LAYOUT_BORDER + offset_x,
+			 1 + DRAG_ICON_LAYOUT_BORDER,
+			 layout);
+
+	gdk_draw_rectangle (drawable,
+			    widget->style->black_gc,
+			    FALSE,
+			    0, 0,
+			    pixmap_width + 1,
+			    pixmap_height + 1);
+
+	g_object_unref (layout);
+
+	g_free (address);
+	g_free (title);
+	g_string_free (text,TRUE);
+
+	return drawable;
+}
+
+static void
+favicon_drag_begin_cb (GtkWidget *widget,
+		       GdkDragContext *context,
+		       EphyLocationEntry *entry)
+{
+	GdkPixmap *pixmap;
+
+	pixmap = favicon_create_drag_pixmap (entry, widget);
+
+	if (pixmap != NULL)
+	{
+		gtk_drag_set_icon_pixmap (context,
+					  gdk_drawable_get_colormap (pixmap),
+					  pixmap, NULL, -2, -2);
+		g_object_unref (pixmap);
+	}
+}
+
 static void
 favicon_drag_data_get_cb (GtkWidget *widget,
 			  GdkDragContext *context,
@@ -587,9 +809,13 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 	priv->icon_ebox = gtk_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (priv->icon_ebox), 2);
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->icon_ebox), FALSE);
+	gtk_widget_add_events (priv->icon_ebox, GDK_BUTTON_PRESS_MASK |
+			      			GDK_BUTTON_RELEASE_MASK |
+						GDK_POINTER_MOTION_HINT_MASK);
 	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
 			     url_drag_types, G_N_ELEMENTS (url_drag_types),
 			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+
 	gtk_tooltips_set_tip (priv->tips, priv->icon_ebox,
 			      _("Drag and drop this icon to create a link to this page"), NULL);
 
@@ -611,6 +837,9 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 
 	g_signal_connect (priv->icon_ebox, "drag-data-get",
 			  G_CALLBACK (favicon_drag_data_get_cb), entry);
+	g_signal_connect (priv->icon_ebox, "drag-begin",
+			  G_CALLBACK (favicon_drag_begin_cb), entry);
+
 	g_signal_connect (priv->lock_ebox, "button-press-event",
 			  G_CALLBACK (lock_button_press_event_cb), entry);
 
@@ -770,6 +999,9 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), text);
 	priv->user_changed = TRUE;
 
+	/* We need to call update_address_state() here, as the 'changed' signal
+	 * may not get called if the user has typed in the exact correct url */
+	update_address_state (entry);
 	update_favicon (entry);
 
 	/* Now restore the selection.
@@ -850,6 +1082,31 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+void
+ephy_location_entry_set_secure (EphyLocationEntry *entry,
+				gboolean secure)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	GtkWidget *widget = GTK_WIDGET (entry);
+	GtkWidget *gentry = ephy_icon_entry_get_entry (entry->priv->icon_entry);
+
+	priv->secure = secure;
+
+	/* We have to set the colour of the GtkEntry in the EphyIconEntry */
+	if (priv->secure && priv->apply_colours)
+	{
+		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, &priv->secure_fg_colour);
+		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, &priv->secure_bg_colour);
+	}
+	else
+	{
+		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, NULL);
+	}
+
+	gtk_widget_queue_draw (widget);
+}
+
 void
 ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
 				   gboolean show_lock)
